import logging

from ConfigParser import *

log = logging.getLogger("parsley.config")

class Config(object):
    def __init__(self):
        self.sections = list()

    def parse_files(self, paths):
        parser = SafeConfigParser()
        found = parser.read(paths)

        for path in found:
            log.info("Read config file '%s'", path)

        if not found:
            log.warn("No config files found at %s", ", ".join(paths))

        sections = ConfigValues()

        for section in self.sections:
            values = section.parse(parser)
            sections[section.name] = values

        return sections

class ConfigSection(object):
    def __init__(self, config, name, strict_section=False):
        assert isinstance(config, Config)

        self.config = config
        self.config.sections.append(self)

        self.name = name
        self.strict_section = strict_section
        self.parameters = list()

    def parse(self, parser):
        in_values = dict()
        out_values = ConfigValues()

        for section in ("common", self.name):
            try:
                items = parser.items(section)
            except NoSectionError:
                if self.strict_section:
                    raise
                continue

            in_values.update(items)

        for param in self.parameters:
            try:
                string = in_values[param.name]
                value = param.unmarshal(string)
            except KeyError:
                value = param.default

            name = param.name.replace("-", "_")
            out_values[name] = value

        return out_values

class ConfigParameter(object):
    def __init__(self, section, name, type):
        assert isinstance(section, ConfigSection)

        self.section = section
        self.section.parameters.append(self)

        self.name = name
        self.type = type

        self.default = None

    def unmarshal(self, string):
        if self.type == bool:
            return string.lower() not in ("f", "false", "no", "0")
        return self.type(string)

class ConfigValues(dict):
    def __getattr__(self, name):
        return self[name]
